{% load sri %}

<!-- Start DataTables ColumnControl CSS -->
{% sri_static "allianceauth/libs/DataTables/Extensions/ColumnControl/1.2.0/css/columnControl.bootstrap5.min.css" %}
<!-- End DataTables ColumnControl CSS -->
