# Generated by Django 4.2.28 on 2026-02-16 12:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("timerboard", "0008_alter_timer_timer_type"),
    ]

    operations = [
        migrations.AlterField(
            model_name="timer",
            name="structure",
            field=models.CharField(
                choices=[
                    ("Other", "Other"),
                    ("Astrahus", "Astrahus"),
                    ("Athanor", "Athanor"),
                    ("Azbel", "Azbel"),
                    ("Pharolux Cyno Beacon", "Cyno Beacon"),
                    ("Tenebrex Cyno Jammer", "Cyno Jammer"),
                    ("Fortizar", "Fortizar"),
                    ("Ansiblex Jump Gate", "Jump Bridge"),
                    ("Keepstar", "Keepstar"),
                    ("Mercenary Den", "Mercenary Den"),
                    ("Metenox Moon Drill", "Moon Drill"),
                    ("Moon Mining Cycle", "Moon Mining Cycle"),
                    ("POCO", "POCO"),
                    ("POS[S]", "POS [S]"),
                    ("POS[M]", "POS [M]"),
                    ("POS[L]", "POS [L]"),
                    ("Raitaru", "Raitaru"),
                    ("Orbital Skyhook", "Skyhook"),
                    ("Sotiyo", "Sotiyo"),
                    ("I-HUB", "Sovereignty Hub"),
                    ("Tatara", "Tatara"),
                ],
                default="Other",
                max_length=254,
            ),
        ),
    ]
