{% load sri %}

<!-- Start DataTables JS -->
{% sri_static "allianceauth/libs/DataTables/2.3.6/js/dataTables.min.js" %}
{% sri_static "allianceauth/libs/DataTables/2.3.6/js/dataTables.bootstrap5.min.js" %}
<!-- End DataTables JS -->
