# Generated by Django 4.2.25 on 2025-10-14 22:02

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("authentication", "0024_alter_userprofile_language"),
    ]

    operations = [
        migrations.AddField(
            model_name="userprofile",
            name="minimize_sidebar",
            field=models.BooleanField(
                default=False,
                help_text="Keep the sidebar menu minimized",
                verbose_name="Minimize Sidebar Menu",
            ),
        ),
    ]
