# Generated by Django 1.10.2 on 2016-12-12 01:00

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('mumble', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelTable(
            name='mumbleuser',
            table=None,
        ),
    ]
