# Generated by Django 4.2.28 on 2026-02-14 13:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("custom_css", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="customcss",
            name="css_compressed",
            field=models.TextField(
                blank=True,
                help_text="This field is automatically generated and should not be edited.",
                null=True,
                verbose_name="Compressed custom CSS",
            ),
        ),
    ]
